/**************************************************************************
 FolderMenu - easy access to project folders from menu.
 
 Copyright (C) 2013 Yu Tang
               Home page: http://sourceforge.jp/users/yu-tang/
               Support center: http://sourceforge.jp/users/yu-tang/pf/

 This file is part of plugin for OmegaT.
 http://www.omegat.org/

 License: GNU GPL version 3 or (at your option) any later version.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

package org.omegat.plugin.foldermenu;

import java.awt.Desktop;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.Arrays;
import java.util.Comparator;
import javax.swing.Icon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.MenuElement;
import javax.swing.MenuSelectionManager;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuKeyEvent;
import javax.swing.event.MenuKeyListener;
import javax.swing.event.MenuListener;
import javax.swing.filechooser.FileSystemView;
import org.omegat.core.Core;
import org.omegat.util.Log;

import static org.omegat.util.Platform.*;
import org.omegat.util.StaticUtils;
import org.openide.awt.Mnemonics;

/**
 *
 * @author Yu-Tang
 */
public class Submenu {

    static {
        fs = FileSystemView.getFileSystemView();
        ff = new FileFilter() {

            @Override
            public boolean accept(File file) {
                return !file.getName().startsWith(".") && !file.isHidden();
            }

        };

        al = new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                open(e.getActionCommand());
            }
            
        };

        mkl = new MenuKeyListener() {

            @Override
            public void menuKeyTyped(MenuKeyEvent e) { /* do nothing */ }

            @Override
            public void menuKeyPressed(MenuKeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    MenuSelectionManager manager = e.getMenuSelectionManager();
                    MenuElement[] selectedPath = manager.getSelectedPath();
                    MenuElement selection = selectedPath[selectedPath.length-1];
                    if (selection instanceof JMenu) {
                        JMenu menu = (JMenu) selection;
                        if (menu.isEnabled()) {
                            manager.clearSelectedPath();
                            open(menu.getActionCommand());
                        }
                    }
                }
            }

            @Override
            public void menuKeyReleased(MenuKeyEvent e) { /* do nothing */ }

        };

        mol = new MouseListener() {

            @Override
            public void mouseClicked(MouseEvent e) {
                JMenu m = (JMenu) e.getSource();
                if (m.isEnabled()) {
                    open(m.getActionCommand());
                }
            }

            @Override
            public void mousePressed(MouseEvent e) { /* do nothing */ }

            @Override
            public void mouseReleased(MouseEvent e) { /* do nothing */ }

            @Override
            public void mouseEntered(MouseEvent e) { /* do nothing */ }

            @Override
            public void mouseExited(MouseEvent e) { /* do nothing */ }
            
        };
        
        mel = new MenuListener() {

            @Override
            public void menuSelected(MenuEvent e) {
                // Lazy create submenus
                JMenu menu = (JMenu) e.getSource();
                if (menu.isEnabled()) {
                    File folder = new File(menu.getActionCommand());
                    File[] filteredListFiles = Submenu.getFilteredListFiles(folder);
                    try {
                        prepare(folder, menu, filteredListFiles);
                    } catch (IOException ex) {
                        Log.log(ex);
                    }    
                }
            }

            @Override
            public void menuDeselected(MenuEvent e) { /* do nothing */ }

            @Override
            public void menuCanceled(MenuEvent e) { /* do nothing */ }
            
        };
        
        comp = new Comparator() {

            @Override
            public int compare(Object o1, Object o2) {
                File f1 = (File) o1;
                File f2 = (File) o2;

                if (f1.isFile() == f2.isFile()) {
                    return f1.getName().compareToIgnoreCase(f2.getName());
                } else {
                    return f1.isFile() ? 1 : -1;
                }
            }
        };
    }
    
    private Submenu() {} // no instanciation, static only.
    
    public static void create(File file, JMenu parent) throws IOException {
        if (file.isDirectory()) {
            File[] filteredListFiles = getFilteredListFiles(file);
            if (filteredListFiles.length > 0) {
                parent.add(createMenu(file));
                return;
            }
        }
        parent.add(createMenuItem(file));
    }
    
    // create Menu only - must have children items
    public static JMenu create(File folder, String label, JMenu parent) throws IOException {
        if (folder.isDirectory()) {
            JMenu m = createMenu(folder, label);
            parent.add(m);
            return m;
        } else {
            return create(label, parent);
        }
    }

    // create Menu only - must have children items
    // Use for create ProjectRoot menu only.
    public static JMenu create(String label, JMenu parent) throws IOException {
        JMenu m = createMenu(label);
        m.setEnabled(false);
        parent.add(m);
        return m;
    }
    
    public static Icon getIcon(File file) {
        return fs.getSystemIcon(file);
    }
        
    public static File[] getFilteredListFiles(File folder) {
        return folder.listFiles(ff);
    }
            
    public static ActionListener getActionListener() {
        return al;
    }
            
    public static MenuKeyListener getMenuKeyListener() {
        return mkl;
    }
    
    public static MouseListener getMouseListener() {
        return mol;
    }
        
    public static MenuListener getMenuListener() {
        return mel;
    }
            
    public static Comparator getComparator() {
        return comp;
    }
    
    public static void open(String path) {
        try {
            switch (getOsType()) {
                case WIN64:
                case WIN32:
                    Desktop.getDesktop().open(new File(path));
                    break;
                case MAC64:
                case MAC32:
                    new ProcessBuilder("open", path).start();
                    break;
                default: // Linux and others
                    new ProcessBuilder("xdg-open", path).start();
                    break;
            }
        } catch (IOException ex) {
            Core.getMainWindow().showMessageDialog(StaticUtils.format(
                    L10n.getErrMsgFileHasNoAssoc(), path));
        }
    }

    public static void associate(JMenu menu, File folder) throws IOException {
        menu.setActionCommand(folder.getCanonicalPath());
        menu.setIcon(Submenu.getIcon(folder));
        menu.setEnabled(true);
    }
    
    public static void removeChildren(JMenu menu) {
        menu.setEnabled(false);
        menu.removeAll();
        menu.setActionCommand("");
        menu.setIcon(null);
    }

    private static JMenuItem createMenuItem(File file) throws IOException {
        JMenuItem m = new JMenuItem(file.getName(), Submenu.getIcon(file));
        m.setActionCommand(file.getCanonicalPath());
        m.addActionListener(getActionListener());
        return m;
    }

    private static JMenuItem createMenuItem(File folder, String label) throws IOException {
        JMenuItem m = new JMenuItem(Submenu.getIcon(folder));
        Mnemonics.setLocalizedText(m, label);
        m.setActionCommand(folder.getCanonicalPath());
        m.addActionListener(getActionListener());
        return m;
    }
    
    private static JMenu createMenu(File folder) throws IOException {
        return createMenu(folder, null);
    }
    
    private static JMenu createMenu(String label) throws IOException {
        return createMenu(null, label);
    }
    
    private static JMenu createMenu(File folder, String label) throws IOException {
        JMenu m;
        if (label == null) {
            m = new JMenu(folder.getName());
        } else {
            m = new JMenu();
            Mnemonics.setLocalizedText(m, label);
        }
        
        m.addMenuListener(getMenuListener());
        m.addMouseListener(getMouseListener());
        
        if (folder != null) {
            m.setIcon(getIcon(folder));
            m.setActionCommand(folder.getCanonicalPath());
            m.setEnabled(true);           
        }

        return m;
    }

    private static void prepare(File folder, JMenu menu, File[] filteredListFiles) throws IOException {
        if (menu.getItemCount() > 0) {
            // submenus are already created
            return;
        }

        menu.setIcon(Submenu.getIcon(folder));
        menu.setActionCommand(folder.getCanonicalPath());
        Arrays.sort(filteredListFiles, getComparator());
        createSubmenus(filteredListFiles, menu);
    }
    
    private static void createSubmenus(File[] files, JMenu parent) {
        //Log.log("createSubmenus...");
        for (File file : files) {
            //Log.log("  " + file.getName());
            try {
                Submenu.create(file, parent);
            } catch (IOException ex) {
                Log.log(ex);
            }
        }
    }
    
    private static final FileSystemView fs;
    private static final FileFilter      ff;
    private static final ActionListener  al;
    private static final MenuKeyListener mkl;
    private static final MouseListener   mol;
    private static final MenuListener    mel;
    private static final Comparator      comp;
}
